/*************************************
*         Copyright 2015 EDF         *
*************************************/
#pragma once

#include "Global.h"
#include <stdexcept>

struct TErrorLevel{
	/**Type numr des messages gr par le gestionnaire de messages*/
	typedef enum EErrorLevel{
		info=0,//!<Messages d'information
		warning=1,//!<Messages d'avertissement de niveau faible
		waring_last=99,//!<Messages d'avertissement de niveau lev
		error//!<Messages d'erreur
	}EErrorLevel;
};
/**Interface de base du gestionnaire de messages.
Cette classe est implmente sous forme d'un singleton par la classe CLogManager.
Le gestionnaire de message sert  enregistrer et mettre en forme les diffrents messages d'erreur ou d'information.
Il peut ventuellement construire un fichier de log.
*/
class PYC_PUBLIC ILogManager
{
	static ILogManager*st_LogManager;
	static std::ostream*st_TrStream;//Stream d'criture de la trace
	ILogManager(ILogManager const&);//Interdit
	ILogManager const&operator=(ILogManager const&);//Interdit
public:
	ILogManager(){}
	virtual ~ILogManager(){}
	/**Classe de gestion des contextes d'mission des messages.
	La construction d'un objet de cette classe incrmente le contexte des messages
	La destruction de l'objet dcrmente le contexte.
	*/
	class PYC_PUBLIC CCtx{
		CCtx(CCtx const&);//Interdit
		CCtx const&operator=(CCtx const&);//Interdit
	public:
		/**Construit un nouveau contexte de message de texte ctx*/
		explicit CCtx(char const*ctx){ILogManager::glLogManager().pushContext(ctx);}
		~CCtx(){ILogManager::glLogManager().popContext();}
		/**Change le contexte de message courant et le remplace par ctx*/
		void changeContext(char const*ctx){ILogManager::glLogManager().popContext();ILogManager::glLogManager().pushContext(ctx);}
	};
	/**Type numr d'encodage des message*/
	typedef enum{
		lm_latin1=0,//!<Encodage standard
		lm_console,//!<Encodage console Windows
		lm_UTF8//!<UTF8
	}TEncoding;
	/**Retourne l'unique gestionnaire de message global*/
	static ILogManager&glLogManager();
	/**Dtruit le manager de message*/
	static void terminate();
	/**Modifie le flux d'criture de la trace*/
	static void setTraceStream(std::ostream*trStream);
	/**Retourne le flux d'criture de la trace*/
	static std::ostream&trStream(){return*st_TrStream;}
	/**Modifie le niveau de restitution locale des messages.*/
	virtual void setDisplayLevel(int level)=0;
	/**Enregistre une erreur systme de code ErrorCode. L'erreur est transforme en texte.*/
	virtual const char *sysError(unsigned long ErrorCode=0)=0;
	/**Enregistre un message de log de niveau level*/
	virtual void msgVerbose(int level,char const*msg)const=0;
	/**Modifie le niveau de prise en compte des messages de log.*/
	virtual void setVerboseLevel(int level)=0;
	/**Enregistre un message d'erreur*/
	virtual TErrorLevel::EErrorLevel msgError(char const*msg)=0;
	/**Lance une exception de message msg*/
	virtual void throwError(char const*msg=0,bool bThrow=true)=0;
	/**enregistre un message d'avertissement de niveau level*/
	virtual TErrorLevel::EErrorLevel msgWarning(char const*msg,TErrorLevel::EErrorLevel level)=0;
	/**Enregistre un message d'information*/
	virtual TErrorLevel::EErrorLevel msgInfo(char const*msg)=0;
	/**Vide l'enregistrement de tous les messages*/
	virtual void clear()=0;
	/**Incrmente le contexte d'mission des messages avec le contexte ctx.*/
	virtual int pushContext(char const*ctx)=0;
	/**Dcrmente le contexte d'mission des messages*/
	virtual int popContext(int toLevel=-1)=0;
	/**Retourne le nombre d'rreurs enregistres*/
	virtual int nbErrors()const=0;
	/**Modifie la valeur de l'encodage souhait*/
	virtual void setExcEncoding(TEncoding enc)=0;
	/**Retourne une chane de caractres contenant l'ensemble des messages enregistrs*/
	virtual char const* messages(TEncoding enc)const=0;
	char const*UTF8Messages()const{return messages(lm_UTF8);}
	/**Imprime les messages enregistrs sur la sortie standard*/
	virtual void printMessages()const=0;
	/**Retourne le temps cpu actuel en millisecondes.
	@param initTime instant initial de mesure du temps cpu
	@param print gre l'impression du temps cpu (=0 pas d'impression, =1 impression console du temps coul depuis initTime, =2 impression message d'info)
	@return l'instant cpu en millisecondes
	*/
	virtual unsigned long long cpuTime(unsigned long long initTime,int print)=0;
	/**Retourne la date actuelle en millisecondes.
	@param initTime date initiale de mesure du temps d'attente
	@param print gre l'impression du temps d'attente (=0 pas d'impression, =1 impression console du temps coul depuis initTime, =2 impression message d'info)
	@return la date actuelle en millisecondes
	*/
	virtual unsigned long long waitTime(unsigned long long initTime,int print)=0;
	/**Retourne vrai si l'exception est une exception lance par ce gestionnaire de messages*/
	virtual bool isLogManagerException(std::exception const&exc)const=0;
	/**Retourne la version de la bibliothque Pycatshoo*/
	virtual char const*version()const=0;
	/**Retourne le copyright de la bibliothque Pycatshoo*/
	virtual char const*copyright()const=0;
	/**Inhibe ou deshinibe l'affichage de la trace.*/
	virtual void inhibateTrace(bool inhib=true)=0;
	/**Retourne l'tat d'hinibition de la trace.*/
	virtual bool traceInhibited()const=0;
};

PYC_PUBLIC std::string formatMsg(char const*format,...);

extern ILogManager*st_LogManager;

