/*************************************
*         Copyright 2015 EDF         *
*************************************/
#pragma once

#include "Global.h"

/**Interface de dfinition d'un gnrateurs de nombres pseudo alatoires.
Cette classe permet de grer l'accs  plusieurs gnrateurs et de dsigner le gnrateur courant grce  des mthodes statiques.
*/
class PYC_PUBLIC IRNG
{
	static IRNG*m_Rngs[];//!<Liste des gnrateurs alatoires disponibles
	IRNG(IRNG const&);//Interdit
	IRNG const&operator=(IRNG const&);//Interdit
protected:
	static IRNG*m_Rng;//!<Gnrateur alatoire courant
	unsigned int max_test;//!<Nombre max de test pour les distributions tronques
public:
	/**Construit une interface de gnration de nombres pseudo alatoires*/
	IRNG():max_test(1000){}
	virtual ~IRNG(){removeRng(*this);}
	/**Demande l'ajout du gnrateur de nom name  la liste des gnrateurs disponibles.
	Ce gnrateur devient le gnrateur courant.*/
	static IRNG*addRng(char const*name);
	/**Ajoute le gnrateur rng aux gnrateurs disponibles.*/
	static void addRng(IRNG&rng);
	/**Supprime le gnrateur rng des gnrateurs disponibles*/
	static void removeRng(IRNG&rng);
	/**Retourne la liste des gnrateurs disponibles*/
	static IRNG*const*rngs(){return m_Rngs;}
	/**Retourne le gnrateur de nom name*/
	static IRNG*get_rng(char const*name);
	/**Dsigne le grrateur de nom name comme le gnrateur courant.
	@return le gnrateur de nom name
	*/
	static IRNG*set_rng(char const*name);
	/**Retourne le gnrateur courant*/
	inline static IRNG&rng(){return*m_Rng;}
	/**Retourne le nom du gnrateur.*/
	virtual char const*name()const=0;
	/**Modifie la graine du gnrateur*/
	virtual void srand(unsigned int the_seed)=0;
	/**Retourne un nombre alatoire dans l'intervalle [0,1]*/
	virtual double drand()=0;
	/**Retourne un nombre alatoire dans l'intervalle [0,1[*/
	virtual double drand_1()=0;
	/**Retourne un nombre alatoire dans l'intervalle ]0,1]*/
	virtual double drand_0()=0;
	/**Retourne un nombre alatoire dans l'intervalle ]0,1[*/
	virtual double drand_0_1()=0;
	/**Retourne un nombre alatoire respectant une loi exponentielle de taux lambda*/
	virtual double exponential(double lambda);
	/**Retourne un nombre alatoire respectant une loi normale de moyenne mu et d'cart type sigma*/
	virtual double normal(double mu,double sigma);
	/**Retourne un nombre alatoire respectant une loi normale de moyenne mu et d'cart type sigma tronque  l'intervalle [min,max]*/
	virtual double normal(double mu,double sigma,double min,double max);
	/**Retourne un nombre alatoire respectant une loi lognormale de moyenne mu et d'cart type sigma*/
	virtual double lognormal(double mu,double sigma);
	/**Retourne un nombre alatoire respectant une loi uniforme sur l'intervalle [min,max]*/
	virtual double uniform(double min,double max);
	/**Retourne un nombre alatoire respectant une loi triangulaire sur l'intervalle [min,max] avec un maximum du triangle en interm*/
	virtual double triangle(double min,double interm,double max);
	/**Retourne un nombre alatoire respectant un loi gamma de paramtres alpha et beta*/
	virtual double gamma(double alpha,double beta);
	virtual double beta(double p,double q,double a,double b);
	virtual double gumbel(double mu,double sigma);
	virtual double frechet(double alpha,double beta);
	virtual double pareto(double alpha,double beta,double delta);
	virtual double weibull(double alpha,double beta);
	/**Retourne vrai si le gnrateur permet d'effectuer rapidement des sauts de N gnrations de nombres*/
	virtual unsigned int can_jump()const=0;
	/**Effectue un saut de size gnrations de nombres*/
	virtual void jump(unsigned long long size)=0;
	/**Fixe la taille des blocs de gnration de nombres (ne fait rien par dfaut)*/
	virtual void set_bloc_size(unsigned int size){};//Ne fait rien par dfaut
	/**Effectue un saut de gnrations de nombres de manire  atteindre le dbut du bloc suivant*/
	virtual void jump(){};//Ne fait rien par dfaut
};
