/*************************************
*         Copyright 2015 EDF         *
*************************************/
#pragma once

#include "Global.h"

#include <complex>

#ifdef WIN32 //Dsactivation du warning 'this': utilis dans la liste des initialiseurs membre de base
#pragma warning (disable : 4355)
#endif

struct _object;
typedef struct _object PyObject;
class CSystem;
class IEquation;
class IInequality;

/**Classe de base de l'ensemble des composants nomms intervenant dans un systme
 La classe maintient une relation avec son parent.
*/
class PYC_PUBLIC CNamedComp
{
	CSystem*m_System;//!<Systme contenant l'lment
	CNamedComp*m_Parent;//!<Parent de l'lment
	char*m_Basename;//!<Nom local de l'lment
	char*m_Name;//!<Nom complet de l'lment
	char*m_Description;//!Description de l'lment
	CNamedComp(CNamedComp const&){}//!<Interdit de copier
	CNamedComp const&operator=(CNamedComp const&){return*this;}//!<Interdit de copier
protected:
	virtual bool verify(){return true;}//!<Mthode de vrification de l'lment appele en dbut de simulation
	CNamedComp(CSystem&sys,char const*name);//!<Construit un composant de nom name directement dans le systme
	CNamedComp(char const*name,CNamedComp*parent=NULL);//!<Construit un composant de nom name
	virtual ~CNamedComp(void){free(m_Basename);free(m_Name);}
public:
	char const*basename()const{return m_Basename;}//!<Retourne le nom local du composant
	void setBasename(char const*bName);//!<Modifie le nom local du composant
	virtual void updateName();//!<Met  jour le nom aprs modification du nom de base ou des parents
	char const*name()const{return m_Name;}//!<Retourne le nom compos du composant construit en cumulant les noms des parents spars par des '.'
	void setDescription(char const*description);//!<Modifie la description du composant
	char const*description()const{return m_Description;}//!<Retourne la description du composant
	virtual void callNamedCompPyMethod(char const*name,PyObject*pVoid)const{}
	virtual void callNamedCompPyMethod(char const*name,PyObject*pVoid,char const*arg)const{}
	virtual std::complex<double>callNamedCompCPyFunction(char const*name,PyObject*pVoid)const{return 0;}
	virtual double callNamedCompDPyFunction(char const*name,PyObject*pVoid,double val)const{return 0;}
	virtual double callNamedCompDPyFunction(char const*name,PyObject*pVoid)const{return 0;}
	virtual float callNamedCompFPyFunction(char const*name,PyObject*pVoid)const{return 0;}
	virtual int callNamedCompIPyFunction(char const*name,PyObject*pVoid)const{return 0;}
	virtual bool callNamedCompBPyFunction(char const*name,PyObject*pVoid)const{return 0;}
	virtual bool callBPyFunction(PyObject*pVoid,float val)const{return false;}
	virtual char const*callNamedCompSPyFunction(char const*name,PyObject*pVoid)const{return "";}
	virtual void callNamedCompPyEquation(char const*name,PyObject*pVoid,IEquation&equ)const{;}
	virtual void callNamedCompPyInequality(char const*name,PyObject*pVoid,IInequality&equ)const{;}
	CNamedComp*parent()const{return m_Parent;}//!<Retourne le parent du composant
	PyTime currentTime()const;//!<Retourne l'instant courant de simulation
	bool isRunning()const;//!<Retourne vrai si la simulation est en cours
	bool isODESolving()const;//!<Retourne vrai si une rsolution d'ODE est en cours
	CSystem&system()const{return*m_System;}//!<Retourne le systme de gestion de la simulation
	int compare(CNamedComp const&other)const;//!<Comparaison en fonction du nom
	bool operator<(CNamedComp const&other)const;//!<Comparaison en fonction du nom
	/**Retourne les noms terminaux du nom compos name ou bien tout le nom terminal si all=vrai.
	fName contient en sortie le premier nom ou tous les premiers noms si all=vrai.
	*/
	static char const*firstName(char const*name,char*fName,bool all=false);
};
