#pragma once
#define BOOST_PYTHON_STATIC_LIB
#include <boost/python/class.hpp>
#include <boost/python/module.hpp>
#include <boost/python/def.hpp>
#include <boost/python/call_method.hpp>
#include <boost/python/overloads.hpp>
#include <boost/python/reference_existing_object.hpp>
#include <boost/python/manage_new_object.hpp>
#include <boost/python/copy_const_reference.hpp>
#include <boost/python/return_value_policy.hpp>
#include <boost/python/pure_virtual.hpp>
#include <boost/python/enum.hpp>
#include <boost/python/list.hpp>
#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/python/suite/indexing/map_indexing_suite.hpp>
#include <boost/python/exception_translator.hpp>
#include <boost/python/wrapper.hpp>

typedef boost::python::override TPythonOverride;

#include <iostream>
#include <string>
#include <vector>
#include "Component.h"
#include "PyFctFilre.h"

boost::python::object IVariable_value(IVariable*var){
	std::type_info const&ti=var->typeInfo();
	if(ti==typeid(bool))
		return boost::python::object(var->bValue());
	if(ti==typeid(int))
		return boost::python::object(var->iValue());
	if(ti==typeid(float))
		return boost::python::object(var->fValue());
	if(ti==typeid(double))
		return boost::python::object(var->dValue());
	if(ti==typeid(std::complex<double>))
		return boost::python::object(var->cValue());
	if(ti==typeid(std::string))
		return boost::python::object(var->sValue());
	ILogManager::glLogManager().throwError(formatMsg("Type inconnu pour la variable %s",var->name()).c_str());
	return boost::python::object();
}

boost::python::object IVariable_initValue(IVariable*var){
	std::type_info const&ti=var->typeInfo();
	if(ti==typeid(bool))
		return boost::python::object(var->bInitValue());
	if(ti==typeid(int))
		return boost::python::object(var->iInitValue());
	if(ti==typeid(float))
		return boost::python::object(var->fInitValue());
	if(ti==typeid(double))
		return boost::python::object(var->dInitValue());
	if(ti==typeid(std::complex<double>))
		return boost::python::object(var->cInitValue());
	if(ti==typeid(std::string))
		return boost::python::object(var->sInitValue());
	ILogManager::glLogManager().throwError(formatMsg("Type inconnu pour la variable %s",var->name()).c_str());
	return boost::python::object();
}

boost::python::object IVarRef_value(IReference*ref,CNX_ID i){
	std::type_info const&ti=ref->typeInfo(i);
	if(ti==typeid(bool))
		return boost::python::object(ref->bValue(i));
	if(ti==typeid(int))
		return boost::python::object(ref->iValue(i));
	if(ti==typeid(float))
		return boost::python::object(ref->fValue(i));
	if(ti==typeid(double))
		return boost::python::object(ref->dValue(i));
	if(ti==typeid(std::complex<double>))
		return boost::python::object(ref->cValue(i));
	if(ti==typeid(std::string))
		return boost::python::object(ref->sValue(i));
	ILogManager::glLogManager().throwError(formatMsg("Type inconnu pour la %dme connexion de la rfrence %s",(int)i,ref->name()).c_str());
	return boost::python::object();
}

/**Surcharge de la classe IDistLaw pour permettre une spcialisation en Python*/
struct PyDistLaw:public IDistLaw,public boost::python::wrapper<IDistLaw>{
	PyDistLaw(PyObject*self,CNamedComp&parent):IDistLaw(self,parent){
		boost::python::detail::initialize_wrapper(self,this);
	}
	char const*name()const override{
		return get_override("name")();
	}
	PyTime delay()const override{
		TPythonOverride f=get_override("delay");
		return f?f():IDistLaw::delay();
	}
	PyTime delayBeyond(PyTime t)const override{
		return get_override("delayBeyond")(t);
	}
	double density(PyTime t)const override{
		return get_override("density")(t);
	}
	double distribution(PyTime t)const override{
		return get_override("distribution")(t);
	}
};

//Wrapper de la classe IDistLaw
void IDistLaw_setPyParameter(IDistLaw*self,char const*name,PyObject*pyObj){
	if(PyCallable_Check(pyObj))
		self->setPyParameter(name,pyObj,0);
	else
		self->setPyParameter(name,NULL,(LAWP_ID)PyLong_AsLong(pyObj));
}
void IDistLaw_setPyParameter(IDistLaw*self,PyObject*pyObj){
	if(PyCallable_Check(pyObj))
		self->setPyParameter(NULL,pyObj,0);
	else{
		boost::python::extract<std::string>str(pyObj);
		if(str.check())
			self->setPyParameter(str().c_str(),NULL,0);
		else{
			boost::python::extract<double>dbl(pyObj);
			if(dbl.check())
				self->setParameter(dbl());
			else
				self->setParameter(*boost::python::extract<IVariable*>(pyObj));
		}
	}
}
void IDistLaw_setPyParameter(IDistLaw*self,PyObject*pyObj,LAWP_ID Id){
	if(PyCallable_Check(pyObj))
		self->setPyParameter(NULL,pyObj,Id);
	else{
		boost::python::extract<std::string>str(pyObj);
		if(str.check())
			self->setPyParameter(str().c_str(),NULL,Id);
		else{
			boost::python::extract<double>dbl(pyObj);
			if(dbl.check())
				self->setParameter(dbl(),Id);
			else
				self->setParameter(*boost::python::extract<IVariable*>(pyObj),Id);
		}
	}
}
void IDistLaw_insertPyParameter(IDistLaw*self,double val){self->insertParameter(val);}
void IDistLaw_insertPyParameter(IDistLaw*self,IVariable&val){self->insertParameter(val);}

//Wrapper de la classe ITransition
void ITransition_setCondition(ITransition*self,PyObject*pyObj,bool negate){
	if(PyCallable_Check(pyObj))
		self->setPyCondition(NULL,pyObj,negate);
	else{
		boost::python::extract<std::string>str(pyObj);
		if(str.check())
			self->setPyCondition(str().c_str(),NULL,negate);
		else//Ca ne peut pas tre un boolen
			self->setCondition(*boost::python::extract<IVariable*>(pyObj),negate);
	}
}
void ITransition_setCondition(ITransition*self,PyObject*pyObj){
	boost::python::extract<bool>bl(pyObj);
	if(bl.check())
		self->setCondition(bl());
	else
		ITransition_setCondition(self,pyObj,false);
}
void ITransition_setCondition(ITransition*self,char const*val,PyObject*pyObj){//On ne sait pas si l'appel sera prioritaire par rapport  (pyObj,negate)
	if(PyCallable_Check(pyObj))
		self->setPyCondition(val,pyObj);
	else{
		boost::python::extract<bool>bl(pyObj);
		if(bl.check())
			self->setPyCondition(val,NULL,bl);
		else
			self->setPyCondition(val,NULL);
	}
}
void ITransition_addSensitiveMethod(ITransition*self,char const*name,PyObject*pVoid){self->addPySensitiveMethod(name,pVoid);}
void ITransition_addSensitiveMethod(ITransition*self,char const*name){self->addPySensitiveMethod(name,(PyObject*)NULL);}
void ITransition_addTarget(ITransition*self,IState*st){self->addTarget(st,TTransType::trans);}

void IPDMPManager_addODEFct(IPDMPManager*self,char const*name,CComponent&comp){self->addPyEquationMethod(name,comp);}
void IPDMPManager_addODEFct(IPDMPManager*self,char const*name,CComponent&comp,PyObject*pyObj){
	if(PyCallable_Check(pyObj))
		self->addPyEquationMethod(name,comp,pyObj);
	else
		self->addPyEquationMethod(name,comp,(int)PyLong_AsLong(pyObj));
}

void IEquation_setCoefficient(IEquation*self,IVariable*p,double c){self->setCoefficient(p,c);}
void IEquation_setConstant(IEquation*self,double c){self->setConstant(c);}

void IVarBase_addPySensitiveMethod(IVarBase*self,char const*name){self->addPySensitiveMethod(name,(PyObject*)NULL,0);}
void IVarBase_addPySensitiveMethod(IVarBase*self,char const*name,int sens){self->addPySensitiveMethod(name,(PyObject*)NULL,sens);}
void IVarBase_addPySensitiveMethod(IVarBase*self,char const*name,PyObject*pVoid){self->addPySensitiveMethod(name,pVoid,0);}

bool IReference_orValue(IReference*self){return self->orValue();}
bool IReference_andValue(IReference*self){return self->andValue();}
double IReference_sumValue(IReference*self){return self->sumValue();}
double IReference_productValue(IReference*self){return self->productValue();}
int IReference_valuesThat(IReference*self,PyObject*pyCond,std::vector<int>&indexes){return self->valuesThat(CPyCondFctFloat(pyCond),indexes);}

using namespace boost::python;

void export_component(){
		
	/**Export de l'numr de dfinition du type de franchissement d'une transition.*/
	enum_<TTransType::ETransType>("TTransType")
		.value("trans",TTransType::trans)
		.value("fault",TTransType::fault)
		.value("rep",TTransType::rep);
		
	/**Export de l'numr de dfinition du type de loi de probabilit.*/
	enum_<TLawType::ELawType>("TLawType")
		.value("inst",TLawType::inst)
		.value("expo",TLawType::expo)
		.value("cstt",TLawType::cstt)
		.value("defer",TLawType::defer)
		.value("cstd",TLawType::cstd)
        .value("pdefer",TLawType::pdefer);

	/**Export de l'numr de dfinition du type d'une variable d'tat.*/
	enum_<TVarType::EVarType>("TVarType")
		.value("t_bool",TVarType::t_bool)
		.value("t_int",TVarType::t_int)
		.value("t_float",TVarType::t_float)
		.value("t_double",TVarType::t_double)
		.value("t_complex",TVarType::t_complex)
		.value("t_string",TVarType::t_string)
		;

	enum_<TModificationMode::EModificationMode>("TModificationMode")
		.value("not_modifiable",TModificationMode::not_modif)
		.value("modifiable",TModificationMode::modif)
		.value("discrete_modification",TModificationMode::discrete_modification)
		.value("continuous_modification",TModificationMode::continuous_modification)
		.value("cont_modifiable",TModificationMode::cont_modif);

	enum_<TMILPAlgorithmType::EMILPAlgorithmType>("TMILPAlgorithmType")
		.value("simplex",TMILPAlgorithmType::simplex)
		.value("interior_point",TMILPAlgorithmType::interior_point)
		.value("milp",TMILPAlgorithmType::milp);

	enum_<TMILPBacktrackingType::EMILPBacktrackingType>("TMILPBacktrackingType")
		.value("depth_first",TMILPBacktrackingType::depth_first)
		.value("breadth_first",TMILPBacktrackingType::breadth_first)
		.value("best_local_bound",TMILPBacktrackingType::best_local_bound)
		.value("best_projection",TMILPBacktrackingType::best_projection);

	enum_<TMILPBranchingType::EMILPBranchingType>("TMILPBranchingType")
		.value("first_frac_variable",TMILPBranchingType::first_frac_variable)
		.value("last_frac_variable",TMILPBranchingType::last_frac_variable)
		.value("most_frac_variable",TMILPBranchingType::most_frac_variable)
		.value("DrTom_heuristic",TMILPBranchingType::DrTom_heuristic)
		.value("pseudo_cost_heuristic",TMILPBranchingType::pseudo_cost_heuristic);

	enum_<TMILPIntPointType::EMILPIntPointType>("TMILPIntPointType")
		.value("none",TMILPIntPointType::none)
		.value("quot_min_degree",TMILPIntPointType::quot_min_degree)
		.value("appr_min_degree",TMILPIntPointType::appr_min_degree)
		.value("sym_appr_min_degree",TMILPIntPointType::sym_appr_min_degree);

	enum_<TMILPMsgLevel::EMILPMsgLevel>("TMILPMsgLevel")
		.value("off",TMILPMsgLevel::off)
		.value("error",TMILPMsgLevel::error)
		.value("on",TMILPMsgLevel::on)
		.value("all",TMILPMsgLevel::all);

	enum_<TMILPOptionsType::EMILPOptionsType>("TMILPOptionsType")
		.value("none",TMILPOptionsType::none)
		.value("pr_steepest_edge",TMILPOptionsType::pr_steepest_edge)
		.value("hr_ratio_test",TMILPOptionsType::hr_ratio_test)
		.value("presolve",TMILPOptionsType::presolve)
		.value("binarize",TMILPOptionsType::binarize)
		.value("sr_heuristic",TMILPOptionsType::sr_heuristic)
		.value("fp_heuristic",TMILPOptionsType::fp_heuristic)
		.value("ps_heuristic",TMILPOptionsType::ps_heuristic)
		.value("gmi_cuts",TMILPOptionsType::gmi_cuts)
		.value("mir_cuts",TMILPOptionsType::mir_cuts)
		.value("cov_cuts",TMILPOptionsType::cov_cuts)
		.value("clq_cuts",TMILPOptionsType::clq_cuts);

	enum_<TMILPPreprocessingType::EMILPPreprocessingType>("TMILPPreprocessingType")
		.value("none",TMILPPreprocessingType::none)
		.value("root",TMILPPreprocessingType::root)
		.value("all",TMILPPreprocessingType::all);

	enum_<TMILPScalingType::EMILPScalingType>("TMILPScalingType")
		.value("geom_mean",TMILPScalingType::geom_mean)
		.value("equilib",TMILPScalingType::equilib)
		.value("near_p2",TMILPScalingType::near_p2)
		.value("none",TMILPScalingType::none)
		.value("auto",TMILPScalingType::automatic);

	enum_<TMILPSimplexType::EMILPSimplexType>("TMILPSimplexType")
		.value("primal",TMILPSimplexType::primal)
		.value("dual",TMILPSimplexType::dual)
		.value("dual_primal",TMILPSimplexType::dual_primal);

	class_< std::vector<IState*> >("VectorSt")
        .def(vector_indexing_suite< std::vector<IState*> >() );
	class_< std::vector<CComponent*> >("VectorComp")
        .def(vector_indexing_suite< std::vector<CComponent*> >() );
	class_< std::vector<std::string> >("VectorString")
        .def(vector_indexing_suite< std::vector<std::string> >() );
	class_< std::vector<ITransition*> >("VectorTrans")
        .def(vector_indexing_suite< std::vector<ITransition*> >() );
	class_< std::vector<IVariable*> >("VectorVar")
        .def(vector_indexing_suite< std::vector<IVariable*> >() );
	class_< std::vector<IAutomaton*> >("VectorAut")
        .def(vector_indexing_suite< std::vector<IAutomaton*> >() );
	class_< std::vector<IReference*> >("VectorRef")
        .def(vector_indexing_suite< std::vector<IReference*> >() );

	class_<CNamedComp,boost::noncopyable>("CNamedComp",no_init)
		.def("basename",&CNamedComp::basename)
		.def("name",&CNamedComp::name)
		.def("parent",&CNamedComp::parent,return_value_policy<reference_existing_object>())
		.def("description",&CNamedComp::description)
		.def("setDescription",&CNamedComp::setDescription)
	;

	/**Export de l'interface IVarBase*/
	class_<IVarBase,bases<CNamedComp>,boost::noncopyable>("IVarBase",no_init)
		.def("setCnctMax",&IVarBase::setCnctMax)
		.def("cnctMax",&IVarBase::cnctMax)
		.def("addSensitiveMethod",static_cast<void(*)(IVarBase*,char const*)>(IVarBase_addPySensitiveMethod))
		.def("addSensitiveMethod",static_cast<void(*)(IVarBase*,char const*,int)>(IVarBase_addPySensitiveMethod))
		.def("addSensitiveMethod",static_cast<void(*)(IVarBase*,char const*,PyObject*)>(IVarBase_addPySensitiveMethod))
		.def("addSensitiveMethod",static_cast<void(IVarBase::*)(char const*,PyObject*,int)>(&IVarBase::addPySensitiveMethod))
		.def("removeSensitiveMethod",&IVarBase::removeSensitiveMethod);

	/**Export de l'interface CMonitored utile pour l'analyseur*/
	class_<IMonitored,IMonitored*,boost::noncopyable>("IMonitored",no_init)
		.def("basename",&IMonitored::basename)
		.def("name",&IMonitored::name)
		.def("parent",&CNamedComp::parent,return_value_policy<reference_existing_object>())
		.def("setTrace",&IMonitored::setTrace)
		.def("setAlwaysMonitored",&IMonitored::setAlwaysMonitored);

	register_ptr_to_python<boost::shared_ptr<IVarBase> >();

	/**Export de l'interface IReference correspondant aux rfrences.*/
	class_<IReference,IReference*,bases<IVarBase>,boost::noncopyable>("IReference",no_init)
		.def("nbCnx",&IReference::cnctCount)
		.def("cnctCount",&IReference::cnctCount)
		.def("cValue",&IReference::cValue)
		.def("dValue",&IReference::dValue)
		.def("fValue",&IReference::fValue)
		.def("iValue",&IReference::iValue)
		.def("bValue",&IReference::bValue)
		.def("sValue",&IReference::sValue)
		.def("value",IVarRef_value)
		.def("reference",&IReference::reference,return_value_policy<reference_existing_object>())
		.def("variable",&IReference::variable,return_value_policy<reference_existing_object>())
		.def("bind",&IReference::bind)
		.def("unbind",static_cast<void(IReference::*)(IVarBase&)>(&IReference::unbind))
		.def("unbind",static_cast<void(IReference::*)(CNX_ID)>(&IReference::unbind))
		.def("takeResource",&IReference::takeResource)
		.def("restoreResource",&IReference::restoreResource)
		.def("sumValue",&IReference::sumValue)
		.def("productValue",&IReference::productValue)
		.def("orValue",&IReference::orValue)
		.def("andValue",&IReference::andValue)
		.def("sumValue",IReference_sumValue)
		.def("productValue",IReference_productValue)
		.def("orValue",IReference_orValue)
		.def("andValue",IReference_andValue)
		.def("valuesThat",&IReference_valuesThat)
		;

	/**Export de l'interface IVariable correspondant aux variables.*/
	class_<IVariable,IVariable*,bases<IMonitored,IVarBase>,boost::noncopyable>("IVariable",no_init)
		.def("cValue",&IVariable::cValue)
		.def("dValue",&IVariable::dValue)
		.def("fValue",&IVariable::fValue)
		.def("iValue",&IVariable::iValue)
		.def("bValue",&IVariable::bValue)
		.def("sValue",&IVariable::sValue)
		.def("value",IVariable_value)
		.def("cInitValue",&IVariable::cInitValue)
		.def("dInitValue",&IVariable::dInitValue)
		.def("fInitValue",&IVariable::fInitValue)
		.def("iInitValue",&IVariable::iInitValue)
		.def("bInitValue",&IVariable::bInitValue)
		.def("sInitValue",&IVariable::sInitValue)
		.def("initValue",IVariable_initValue)
		.def("setValue",static_cast<void(IVariable::*)(std::complex<double>)>(&IVariable::setValue))
		.def("setValue",static_cast<void(IVariable::*)(double)>(&IVariable::setValue))
		.def("setValue",static_cast<void(IVariable::*)(float)>(&IVariable::setValue))
		.def("setValue",static_cast<void(IVariable::*)(bool)>(&IVariable::setValue))
		.def("setValue",static_cast<void(IVariable::*)(int)>(&IVariable::setValue))
		.def("setValue",static_cast<void(IVariable::*)(char const*)>(&IVariable::setValue))
		.def("setCValue",static_cast<void(IVariable::*)(std::complex<double>)>(&IVariable::setValue))
		.def("setDValue",static_cast<void(IVariable::*)(double)>(&IVariable::setValue))
		.def("setFValue",static_cast<void(IVariable::*)(float)>(&IVariable::setValue))
		.def("setIValue",static_cast<void(IVariable::*)(int)>(&IVariable::setValue))
		.def("setBValue",static_cast<void(IVariable::*)(bool)>(&IVariable::setValue))
		.def("setSValue",static_cast<void(IVariable::*)(char const*)>(&IVariable::setValue))
		.def("setDvdtODE",&IVariable::setDvdtODE)
		.def("setModifiable",&IVariable::setModifiable)
		.def("setInterpolated",&IVariable::setInterpolated)
		.def("isInterpolated",&IVariable::isInterpolated)
		.def("integrate",&IVariable::integrate)
		.def("setReinitialized",&IVariable::setReinitialized)
		.def("derivative",&IVariable::derivative)
		.def("dtDerivative",&IVariable::dtDerivative)
		;

	/**Export de l'interface IMessageBox correspondant aux botes de messages.*/
	class_<IMessageBox,IMessageBox*,bases<CNamedComp>,boost::noncopyable>("IMessageBox",no_init)
		.def("addExport",&IMessageBox::addExport)
		.def("exportedVariable",&IMessageBox::exportedVariable,return_value_policy<reference_existing_object>())
		.def("addImport",&IMessageBox::addImport)
		.def("addOptionalImport",&IMessageBox::addOptionalImport)
		.def("connectTo",static_cast<void(IMessageBox::*)(IMessageBox&,double)>(&IMessageBox::connectTo))
		.def("connectTo",static_cast<void(IMessageBox::*)(IMessageBox&)>(&IMessageBox::connectTo))
		.def("disconnectFrom",&IMessageBox::disconnectFrom)
		.def("cnctCount",&IMessageBox::cnctCount)
		.def("component",&IMessageBox::component,return_value_policy<reference_existing_object>())
		.def("cnct",&IMessageBox::cnct,return_value_policy<reference_existing_object>())
		.def("weight",&IMessageBox::weight)
		.def("cValue",&IMessageBox::cValue)
		.def("dValue",&IMessageBox::dValue)
		.def("fValue",&IMessageBox::fValue)
		.def("iValue",&IMessageBox::iValue)
		.def("bValue",&IMessageBox::bValue)
		.def("sValue",&IMessageBox::sValue)
		;

	/**Export de l'interface IDistLaw*/
	class_<IDistLaw,PyDistLaw,boost::noncopyable>("IDistLaw",init<CNamedComp&>())
		.def("name",&IDistLaw::name)
		.def("delay",&IDistLaw::delay)
		.def("delayBeyond",&IDistLaw::delayBeyond)
		.def("density",&IDistLaw::density)
		.def("distribution",&IDistLaw::distribution)
		.def("index",&IDistLaw::index)
		.def("nbIndex",&IDistLaw::nbIndex)
		.def("nbParam",&IDistLaw::nbParam)
		.def("setParameter",static_cast<void(IDistLaw::*)(char const*,PyObject*,LAWP_ID)>(&IDistLaw::setPyParameter))
		.def("setParameter",static_cast<void(*)(IDistLaw*,PyObject*)>(IDistLaw_setPyParameter))
		.def("setParameter",static_cast<void(*)(IDistLaw*,PyObject*,LAWP_ID)>(IDistLaw_setPyParameter))
		.def("setParameter",static_cast<void(*)(IDistLaw*,char const*,PyObject*)>(IDistLaw_setPyParameter))
		.def("insertParameter",static_cast<void(IDistLaw::*)(IVariable&,LAWP_ID)>(&IDistLaw::insertParameter))
		.def("insertParameter",static_cast<void(IDistLaw::*)(double,LAWP_ID)>(&IDistLaw::insertParameter))
		.def("insertParameter",static_cast<void(*)(IDistLaw*,IVariable&)>(IDistLaw_insertPyParameter))
		.def("insertParameter",static_cast<void(*)(IDistLaw*,double)>(IDistLaw_insertPyParameter))
		.def("newLaw",static_cast<IDistLaw*(*)(CNamedComp&,TLawType::ELawType,IVariable&)>(&IDistLaw::newLaw),return_value_policy<reference_existing_object>())
		.def("newLaw",static_cast<IDistLaw*(*)(CNamedComp&,TLawType::ELawType,double)>(&IDistLaw::newLaw),return_value_policy<reference_existing_object>())
		.def("newLaw",static_cast<IDistLaw*(*)(CNamedComp&,TLawType::ELawType,char const*)>(&IDistLaw::newPyLaw),return_value_policy<reference_existing_object>())
		.def("newLaw",static_cast<IDistLaw*(*)(CNamedComp&,TLawType::ELawType,char const*,PyObject*)>(&IDistLaw::newPyLaw),return_value_policy<reference_existing_object>()).staticmethod("newLaw")
		.def("parameter",&IDistLaw::parameter)
		;

	/**Export de l'interface ITransition.*/
	class_<ITransition,ITransition*,bases<CNamedComp>,boost::noncopyable>("ITransition",no_init)
		.def("setInterruptible",&ITransition::setInterruptible)
		.def("interruptible",&ITransition::interruptible)
		.def("setModifiable",&ITransition::setModifiable)
		.def("modifiable",&ITransition::modifiable)
		.def("startState",&ITransition::startState,return_value_policy<reference_existing_object>())
		.def("addTarget",&ITransition::addTarget)
		.def("addTarget",&ITransition_addTarget)
		.def("insertTarget",&ITransition::insertTarget)
		.def("targetCount",&ITransition::targetCount)
		.def("getTarget",&ITransition::getTarget,return_value_policy<reference_existing_object>())
		.def("getTargetType",&ITransition::getTargetType)
		.def("setCondition",static_cast<void(ITransition::*)(char const*,PyObject*,bool)>(&ITransition::setPyCondition))
		.def("setCondition",static_cast<void(*)(ITransition*,char const*,PyObject*)>(ITransition_setCondition))
		.def("setCondition",static_cast<void(*)(ITransition*,PyObject*,bool)>(ITransition_setCondition))
		.def("setCondition",static_cast<void(*)(ITransition*,PyObject*)>(ITransition_setCondition))
		.def("getCondition",&ITransition::getCondition)
		.def("setDistLaw",static_cast<void(ITransition::*)(IDistLaw*)>(&ITransition::setDistLaw),return_internal_reference<1,with_custodian_and_ward<1,2> >())
		.def("setDistLaw",static_cast<IDistLaw*(ITransition::*)(TLawType::ELawType,double)>(&ITransition::setDistLaw),return_value_policy<reference_existing_object>())
		.def("setDistLaw",static_cast<IDistLaw*(ITransition::*)(TLawType::ELawType,IVariable&)>(&ITransition::setDistLaw),return_value_policy<reference_existing_object>())
		.def("setDistLaw",static_cast<IDistLaw*(ITransition::*)(TLawType::ELawType,char const*name,PyObject*)>(&ITransition::setPyDistLaw),return_value_policy<reference_existing_object>())
		.def("setDistLaw",static_cast<IDistLaw*(ITransition::*)(TLawType::ELawType,char const*name)>(&ITransition::setPyDistLaw),return_value_policy<reference_existing_object>())
		.def("distLaw",&ITransition::distLaw,return_value_policy<reference_existing_object>())
		.def("addSensitiveMethod",static_cast<void(*)(ITransition*self,char const*)>(ITransition_addSensitiveMethod))
		.def("addSensitiveMethod",static_cast<void(*)(ITransition*self,char const*,PyObject*)>(ITransition_addSensitiveMethod))
		.def("addSensitiveMethod",&ITransition::addPySensitiveMethod)
		.def("removeSensitiveMethod",&ITransition::removeSensitiveMethod)
		.def("inhibateTarget",static_cast<void(ITransition::*)(IState*,bool)>(&ITransition::inhibateTarget))
		.def("inhibateTarget",static_cast<void(ITransition::*)(int,bool)>(&ITransition::inhibateTarget))
		.def("firedState",&ITransition::firedState)
		.def("startTime",&ITransition::startTime)
		.def("endTime",&ITransition::endTime)
		.def("setMonitoredOutStateMask",&ITransition::setMonitoredOutStateMask)
		.def("monitoredOutStateMask",&ITransition::monitoredOutStateMask)
		;

	/**Export de l'interface IState.*/
	class_<IState,IState*,bases<IVariable>,boost::noncopyable>("IState",no_init)
		.def("addTransition",&IState::addTransition,return_value_policy<reference_existing_object>())
		.def("isActive",&IState::isActive)
		.def("index",&IState::index)
		.def("exitTime",&IState::exitTime)
		.def("automaton",&IState::automaton,return_value_policy<reference_existing_object>());

	/**Export de l'interface IAutomaton.*/
	class_<IAutomaton,IAutomaton*,bases<IVariable>,boost::noncopyable>("IAutomaton",no_init)
		.def("addState",&IAutomaton::addState,return_value_policy<reference_existing_object>())
		.def("initState",&IAutomaton::initState,return_value_policy<reference_existing_object>())
		.def("states",&IAutomaton::states)
		.def("setInitState",&IAutomaton::setInitState)
		.def("currentIndex",&IAutomaton::currentIndex)
		.def("currentState",&IAutomaton::currentState,return_value_policy<reference_existing_object>());

	/**Export de l'interface IPDMPManager.*/
	class_<IPDMPManager,bases<CNamedComp>,boost::noncopyable>("IPDMPManager",no_init)
		.def("setDt",&IPDMPManager::setDt)
		.def("getDt",&IPDMPManager::getDt)
		.def("setDtMin",&IPDMPManager::setDtMin)
		.def("setDtMax",&IPDMPManager::setDtMax)
		.def("setDtCond",&IPDMPManager::setDtCond)
		.def("setDtMem",&IPDMPManager::setDtMem)
		.def("setAlwaysMem",&IPDMPManager::setAlwaysMem)
		.def("addODEVariable",&IPDMPManager::addODEVariable)
		.def("addExplicitVariable",&IPDMPManager::addExplicitVariable)
		.def("addBeginMethod",static_cast<void(IPDMPManager::*)(char const*,CComponent&)>(&IPDMPManager::addPyBeginMethod))
		.def("addBeginMethod",static_cast<void(IPDMPManager::*)(char const*,CComponent&,PyObject*)>(&IPDMPManager::addPyBeginMethod))
		.def("addEndMethod",static_cast<void(IPDMPManager::*)(char const*,CComponent&)>(&IPDMPManager::addPyEndMethod))
		.def("addEndMethod",static_cast<void(IPDMPManager::*)(char const*,CComponent&,PyObject*)>(&IPDMPManager::addPyEndMethod))
		.def("addEquationMethod",static_cast<void(IPDMPManager::*)(char const*,CComponent&,int)>(&IPDMPManager::addPyEquationMethod))
		.def("addEquationMethod",static_cast<void(IPDMPManager::*)(char const*,CComponent&,PyObject*,int)>(&IPDMPManager::addPyEquationMethod))
		.def("addEquationMethod",static_cast<void(*)(IPDMPManager*,char const*,CComponent&)>(IPDMPManager_addODEFct))
		.def("addEquationMethod",static_cast<void(*)(IPDMPManager*,char const*,CComponent&,PyObject*)>(IPDMPManager_addODEFct))
		.def("addWatchedTransition",static_cast<void(IPDMPManager::*)(ITransition&)>(&IPDMPManager::addWatchedTransition))
		.def("addCondition",static_cast<void(IPDMPManager::*)(char const*,CComponent&)>(&IPDMPManager::addPyCondition))
		.def("addCondition",static_cast<void(IPDMPManager::*)(char const*,CComponent&,PyObject*)>(&IPDMPManager::addPyCondition))
		.def("addBoundaryCheckerMethod",static_cast<void(IPDMPManager::*)(char const*,CComponent&)>(&IPDMPManager::addPyCondition))
		.def("addBoundaryCheckerMethod",static_cast<void(IPDMPManager::*)(char const*,CComponent&,PyObject*)>(&IPDMPManager::addPyCondition))
		.def("setSchema",static_cast<void(IPDMPManager::*)(TSchemaType::ESchemaType)>(&IPDMPManager::setSchema))
		.def("setTrace",&IPDMPManager::setTrace);

	/**Export de l'interface IEquation*/
	class_<IEquation,boost::noncopyable>("IEquation",no_init)
		.def("setCoefficient",&IEquation::setCoefficient)
		.def("setCoefficient",IEquation_setCoefficient)
		.def("setConstant",&IEquation::setConstant)
		.def("setConstant",IEquation_setConstant);

	/**Export de l'interface ISLEManager.*/
	class_<ISLEManager,bases<CNamedComp>,boost::noncopyable>("ISLEManager",no_init)
		.def("addEquation",static_cast<void(ISLEManager::*)(char const*,CNamedComp&,PyObject*)>(&ISLEManager::addPyEquation))
		.def("addEquation",static_cast<void(ISLEManager::*)(char const*,CNamedComp&)>(&ISLEManager::addPyEquation))
		.def("addVariable",&ISLEManager::addVariable)
		.def("print",&ISLEManager::print)
		.def("solve",&ISLEManager::solve);

	/**Export de l'interface IInequality*/
	class_<IInequality,boost::noncopyable>("IInequality",no_init)
		.def("setCoefficient",&IInequality::setCoefficient)
		.def("setLimits",&IInequality::setLimits)
		.def("setMax",&IInequality::setMax)
		.def("setMin",&IInequality::setMin);

	/**Export de l'interface IMILPManager.*/
	class_<IMILPManager,bases<CNamedComp>,boost::noncopyable>("IMILPManager",no_init)
		.def("addInequality",static_cast<void(IMILPManager::*)(char const*,CNamedComp&,PyObject*)>(&IMILPManager::addPyInequality))
		.def("addEquation",static_cast<void(IMILPManager::*)(char const*,CNamedComp&)>(&IMILPManager::addPyInequality))
		.def("addVariable",static_cast<void(IMILPManager::*)(IVariable&,double)>(&IMILPManager::addVariable))
		.def("addVariable",static_cast<void(IMILPManager::*)(IVariable&)>(&IMILPManager::addVariable))
		.def("setVariableLimits",&IMILPManager::setVariableLimits)
		.def("setVariableMin",&IMILPManager::setVariableMin)
		.def("setVariableMax",&IMILPManager::setVariableMax)
		.def("setVariableObjCoef",&IMILPManager::setVariableObjCoef)
		.def("setObjectiveVar",&IMILPManager::setObjectiveVar)
		.def("setMaximize",&IMILPManager::setMaximize)
		.def("setAlgorithm",&IMILPManager::setAlgorithm)
		.def("setMsgOptions",&IMILPManager::setMsgOptions)
		.def("setScaling",&IMILPManager::setScaling)
		.def("setOptions",&IMILPManager::setOptions)
		.def("setOption",&IMILPManager::setOption)
		.def("setTimeLimit",&IMILPManager::setTimeLimit)
		.def("setIterLimit",&IMILPManager::setIterLimit)
		.def("addOptions",&IMILPManager::addOptions)
		.def("delOptions",&IMILPManager::delOptions)
		.def("setSimplexOpt",&IMILPManager::setSimplexOpt)
		.def("setIntPointOpt",&IMILPManager::setIntPointOpt)
		.def("setBacktracking",&IMILPManager::setBacktracking)
		.def("setBranching",&IMILPManager::setBranching)
		.def("setPreprocessing",&IMILPManager::setPreprocessing)
		.def("objectiveValue",&IMILPManager::objectiveValue)
		.def("print",&IMILPManager::print)
		.def("solve",&IMILPManager::solve);

	/**Export de la classe Component sans possibilit de modification.*/
	class_<CComponent,CComponent*,bases<CNamedComp>,boost::noncopyable>("IComponent", no_init)
		.def("getMessageBox",&CComponent::getMessageBox,return_value_policy<reference_existing_object>())
		.def("getPDMPManager",&CComponent::getPDMPManager,return_value_policy<reference_existing_object>())
		.def("getMILPManager",&CComponent::getMILPManager,return_value_policy<reference_existing_object>())
		.def("getSLEManager",&CComponent::getSLEManager,return_value_policy<reference_existing_object>())
		.def("getAutomaton",&CComponent::getAutomaton,return_value_policy<reference_existing_object>())
		.def("getAutomata",&CComponent::getAutomata)
		.def("getComponent",&CComponent::getComponent,return_value_policy<reference_existing_object>())
		.def("getComponents",&CComponent::getComponents)
		.def("getReference",&CComponent::getReference,return_value_policy<reference_existing_object>())
		.def("getReferences",&CComponent::getReferences)
		.def("getVariable",&CComponent::getVariable,return_value_policy<reference_existing_object>())
		.def("getVariables",&CComponent::getVariables)
		.def("getState",&CComponent::getState,return_value_policy<reference_existing_object>())
		.def("getStates",&CComponent::getStates)
		.def("getTransition",&CComponent::getTransition,return_value_policy<reference_existing_object>())
		.def("isInitState",&CComponent::isInitState)
		.def("setInitState",static_cast<void(CComponent::*)(char const*)>(&CComponent::setInitState))
		.def("setInitState",static_cast<void(CComponent::*)(IState*)>(&CComponent::setInitState))
		.def("currentTime",&CComponent::currentTime)
		.def("deleteCmp",&CComponent::deleteCmp)
		.def("className",&CComponent::className)
        ;
}
