# -*- coding: iso-8859-15 -*-
"""
Copyright EDF 2017
@author: Hassane CHRAIBI - hassane.chraibi@edf.fr
"""

from Util import *                                                                              #S1


class Tank(Pyc.CComponent):                                                                     #S1
    def __init__(self, name):                                                                   #S1
        Pyc.CComponent.__init__(self, name)                                                     #S1

        self.po_area           = self.addVariable("area"          , Pyc.TVarType.t_double, 1.)     #S1
        self.po_level          = self.addVariable("level"         , Pyc.TVarType.t_double, 7.)     #S1
        self.po_temperature    = self.addVariable("temperature"   , Pyc.TVarType.t_double, 31.)    #S5

        self.pi_inFlow         = self.addReference("inFlow")                                    #S1
        self.pi_outFlow        = self.addReference("outFlow")                                   #S2a
        self.pi_temperature    = self.addReference("temperature")                               #S5
        self.pi_power          = self.addReference("power")                                     #S7

        self.po_hotTemperature = self.addVariable("hotTemperature", Pyc.TVarType.t_double, 100.)   #S8c
        self.po_overFlowLevel  = self.addVariable("overFlowLevel" , Pyc.TVarType.t_double, 10.)    #S8c
        self.po_dryOutLevel    = self.addVariable("dryOutLevel"   , Pyc.TVarType.t_double, 4.)     #S8c


        self.addMessageBox("Pump")                                                              #S1
        self.addMessageBoxImport("Pump", self.pi_inFlow 	, "flow")                           #S1
        self.addMessageBoxImport("Pump", self.pi_temperature, "temperature")                    #S5
        self.addMessageBoxExport("Pump", self.po_level      , "level")                          #S6a
        self.addMessageBoxExport("Pump", self.po_temperature, "temperature")                    #S8b

        self.addMessageBox("Valve")                                                             #S2a
        self.addMessageBoxImport("Valve", self.pi_outFlow 	 , "flow")                          #S2a
        self.addMessageBoxExport("Valve", self.po_level      , "level")                         #S6a
        self.addMessageBoxExport("Valve", self.po_temperature, "temperature")                   #S8b

        self.addMessageBox("Fuel")                                                              #S7
        self.addMessageBoxImport("Fuel", self.pi_power, "power")                                #S7

        self.levelAutomaton = self.addAutomaton("LevelAutomaton")                               #S8c
        self.levelOK  = self.addState("LevelAutomaton", "LevelOK" , 0)                          #S8c
        self.overFlow = self.addState("LevelAutomaton", "OverFlow", 1)                          #S8c
        self.dryOut   = self.addState("LevelAutomaton", "DryOut"  , 2)                          #S8c
        self.levelAutomaton.setInitState(self.levelOK)                                          #S8c

        trans = self.levelOK.addTransition("LOK_to_OVF")                                        #S8c
        trans.setCondition("overFlowCondition", self.overFlowCondition, False)                  #S8c
        trans.addTarget(self.overFlow, Pyc.TTransType.trans)                                    #S8c

        trans = self.overFlow.addTransition("OVF_to_LOK")                                       #S8c
        trans.setCondition("overFlowCondition", self.overFlowCondition, True)                   #S8c
        trans.addTarget(self.levelOK, Pyc.TTransType.trans)                                     #S8c

        trans = self.levelOK.addTransition("LOK_to_DO")                                         #S8c
        trans.setCondition("dryOutCondition", self.dryOutCondition, False)                      #S8c
        trans.addTarget(self.dryOut, Pyc.TTransType.trans)                                      #S8c

        trans = self.dryOut.addTransition("DO_to_LOK")                                          #S8c
        trans.setCondition("dryOutCondition", self.dryOutCondition, True)                       #S8c
        trans.addTarget(self.levelOK, Pyc.TTransType.trans)                                     #S8c

        self.temperaturAutomaton = self.addAutomaton("TemperatureAutomaton")                     #S8c
        self.temperatureOK  = self.addState("TemperatureAutomaton", "TemperatureOK" , 0)         #S8c
        self.hotTemperature = self.addState("TemperatureAutomaton", "HotTemperature", 1)         #S8c
        self.setInitState("TemperatureOK")

        trans = self.temperatureOK.addTransition("TOK_to_THeigh")                               #S8c
        trans.setCondition("hotTemperatureCondition", self.hotTemperatureCondition, False)      #S8c
        trans.addTarget(self.hotTemperature, Pyc.TTransType.trans)                              #S8c

        trans = self.hotTemperature.addTransition("THeigh_to_TOK")                              #S8c
        trans.setCondition("hotTemperatureCondition", self.hotTemperatureCondition, True)       #S8c
        trans.addTarget(self.temperatureOK, Pyc.TTransType.trans)                               #S8c

        self.addPDMPManager			     ("pdmpManager")                                        #S1
        self.addPDMPEquationMethod       ("pdmpManager", "odeMethod"      , self.odeMethod)     #S1
        self.addPDMPBoundaryCheckerMethod("pdmpManager", "boundaryChecker", self.boundaryChecker) #S8c
        self.addPDMPODEVariable		     ("pdmpManager", self.po_level)                         #S1
        self.addPDMPODEVariable		     ("pdmpManager", self.po_temperature)                   #S5

    def odeMethod(self):                                                                        #S1
        iFlow    = self.pi_inFlow.sumValue()                                                    #S1
        oFlow    = self.pi_outFlow.sumValue()                                                   #S2a

        self.po_level.setDvdtODE((iFlow - oFlow) / self.po_area.dValue())                       #S1 ==> S2a

        sumiFiT = 0                                                                             #S5
        for i in range(self.pi_inFlow.nbCnx()):                                                 #S5
            sumiFiT = sumiFiT + self.pi_inFlow.dValue(i) * \
                                (self.pi_temperature.dValue(i) - self.po_temperature.value())   #S5

        self.po_temperature.setDvdtODE(                                                         #S5
            (sumiFiT + self.pi_power.dValue(0)) /                                               #S5 ==> S7
            (self.po_level.dValue() * self.po_area.dValue())                                    #S5
                                      )                                                         #S5


    def boundaryChecker(self):                                                                                    #S8c
        if ( (self.levelOK.isActive() and self.po_level.dValue() > self.po_overFlowLevel.dValue()) or             #S8c
             (self.levelOK.isActive() and self.po_level.dValue() < self.po_dryOutLevel.dValue()) or               #S8c
             (self.overFlow.isActive() and self.po_level.dValue() <= self.po_overFlowLevel.dValue()) or           #S8c
             (self.dryOut.isActive() and self.po_level.dValue() >= self.po_dryOutLevel.dValue()) or               #S8c
             (self.temperatureOK.isActive() and self.po_temperature.dValue() > self.po_hotTemperature.dValue()) or#S8c
             (self.hotTemperature.isActive() and self.po_temperature.dValue() <= self.po_hotTemperature.dValue()) #S8c
           ) :                                                                                                    #S8c
            return -1. # STOP                                                                                     #S8c
        else :                                                                                                    #S8c
            return 1. # CONTINUE                                                                                  #S8c


    def overFlowCondition(self):                                                                #S8c
        return (self.po_level.dValue() > self.po_overFlowLevel.dValue())                        #S8c


    def dryOutCondition(self):                                                                  #S8c
        return (self.po_level.dValue() < self.po_dryOutLevel.dValue())                          #S8c


    def hotTemperatureCondition(self):                                                          #S8c
        return (self.po_temperature.dValue() > self.po_hotTemperature.dValue())                 #S8c

########################################################################################
class HydroDevice(Pyc.CComponent):                                                              #S2b
    def __init__(self, name):                                                                   #S2b
        Pyc.CComponent.__init__(self, name)                                                     #S2b

        self.po_nominalFlow       = self.addVariable("nominalFlow", Pyc.TVarType.t_double, 1.5)    #S3
        self.po_flow              = self.addVariable("flow"       , Pyc.TVarType.t_double, 1.5)    #S2b

        self.po_lambda0           = self.addVariable("lambda0"    , Pyc.TVarType.t_double, 0.001)  #S8b
        self.po_lambda            = self.addVariable("lambda"     , Pyc.TVarType.t_double, 0.001)  #S4

        self.po_levelMax          = self.addVariable("levelMax"   , Pyc.TVarType.t_double, 8.)     #S6a
        self.po_levelMin          = self.addVariable("levelMin"   , Pyc.TVarType.t_double, 6.)     #S6a

        self.pi_tankLevel         = self.addReference("tankLevel")                              #S6a
        self.pi_tankTemperature   = self.addReference("tankTemperature")                        #S8b



        self.addMessageBox("Tank")                                                              #S2b
        self.addMessageBoxExport("Tank", self.po_flow 	  , "flow")                             #S2b
        self.addMessageBoxImport("Tank", self.pi_tankLevel, "level")                            #S6a
        self.addMessageBoxImport("Tank", self.pi_tankTemperature, "temperature")                #S8b

        self.automaton  = self.addAutomaton("FuncAutomation")                                   #S3
        self.openState  = self.addState("FuncAutomation", "OPEN" , 0)                           #S3
        self.closeState = self.addState("FuncAutomation", "CLOSE", 1)                           #S3
        self.stuckOpenState  = self.addState("FuncAutomation", "STUCKOPEN" , 2)                 #S4
        self.stuckCloseState = self.addState("FuncAutomation", "STUCKCLOSE", 3)                 #S4
        self.automaton.setInitState(self.openState)                                             #S3
        self.automaton.addSensitiveMethod("updateFlow", self.updateFlow)                        #S3

        trans = self.openState.addTransition("OPEN_to_CLOSE")                                   #S3
        trans.setCondition("closeCondition", self.closeCondition, False)                        #S3 ==> S6a
        trans.addTarget(self.closeState, Pyc.TTransType.trans)                                  #S3

        trans = self.closeState.addTransition("CLOSE_to_OPEN")                                  #S3
        trans.setCondition("openCondition", self.openCondition, False)                          #S3 ==> S6a
        trans.addTarget(self.openState, Pyc.TTransType.trans)                                   #S3

        trans = self.openState.addTransition("OPEN_to_STUCKOPEN")                               #S4
        trans.setDistLaw(Pyc.IDistLaw.newLaw(self, Pyc.TLawType.expo, self.po_lambda))          #S4
        trans.addTarget(self.stuckOpenState, Pyc.TTransType.fault)                              #S4
        trans.setModifiable(Pyc.TModificationMode.continuous_modification)                      #S8b

        trans = self.openState.addTransition("OPEN_to_STUCKCLOSE")                              #S4
        trans.setDistLaw(Pyc.IDistLaw.newLaw(self, Pyc.TLawType.expo, self.po_lambda))          #S4
        trans.addTarget(self.stuckCloseState, Pyc.TTransType.fault)                             #S4
        trans.setModifiable(Pyc.TModificationMode.continuous_modification)                      #S8b

        trans = self.closeState.addTransition("CLOSE_to_STUCKOPEN")                             #S4
        trans.setDistLaw(Pyc.IDistLaw.newLaw(self, Pyc.TLawType.expo, self.po_lambda))          #S4
        trans.addTarget(self.stuckOpenState, Pyc.TTransType.fault)                              #S4
        trans.setModifiable(Pyc.TModificationMode.continuous_modification)                      #S8b

        trans = self.closeState.addTransition("CLOSE_to_STUCKCLOSE")                            #S4
        trans.setDistLaw(Pyc.IDistLaw.newLaw(self, Pyc.TLawType.expo, self.po_lambda))          #S4
        trans.addTarget(self.stuckCloseState, Pyc.TTransType.fault)                             #S4
        trans.setModifiable(Pyc.TModificationMode.continuous_modification)                      #S8b

        self.addPDMPManager		         ("pdmpManager")                                            #S6b
        self.addPDMPBoundaryCheckerMethod("pdmpManager", "boundaryChecker", self.boundaryChecker)   #S6b
        self.addPDMPEquationMethod       ("pdmpManager", "odeMethod"      , self.odeMethod, 0)      #S8b
        self.addPDMPExplicitVariable     ("pdmpManager", self.po_lambda)                            #S8b

        self.addStartMethod("updateFlow", self.updateFlow)                                      #S3

    def updateFlow(self):                                                                       #S3
        if self.openState.isActive() or self.stuckOpenState.isActive():                         #S3 ==> S4
            self.po_flow.setValue(self.po_nominalFlow.dValue())                                 #S3
        else :                                                                                  #S3
            self.po_flow.setValue(0)                                                            #S3

    def odeMethod(self):                                                                        #S8b
        b1 = 3.0295                                                                             #S8b
        b2 = 0.7578                                                                             #S8b
        bc = 0.05756                                                                            #S8b
        bd = 0.2301                                                                             #S8b
        lmbda = self.po_lambda0.value()*\
                 ( b1 * np.exp(+ bc * (self.pi_tankTemperature.value(0) - 20)) +               #S8b
                   b2 * np.exp(- bd * (self.pi_tankTemperature.value(0) - 20))                 #S8b
                 ) / (b1 + b2)

        self.po_lambda.setDValue(lmbda)


########################################################################################
class Pump(HydroDevice):                                                                        #S2a ==> S2b
    def __init__(self, name):                                                                   #S1
        HydroDevice.__init__(self, name)                                                        #S2a ==> S2b


        self.po_temperature = self.addVariable("temperature", Pyc.TVarType.t_double, 15.)          #S5

        self.addMessageBoxExport("Tank", self.po_temperature, "temperature")                    #S5


    def openCondition(self):                                                                    #S6a
        return (self.pi_tankLevel.dValue(0) < self.po_levelMin.dValue())                        #S6a

    def closeCondition(self):                                                                   #S6a
        return (self.pi_tankLevel.dValue(0) > self.po_levelMax.dValue())                        #S6a

    def boundaryChecker(self):                                                                      #S6b
        if self.stuckOpenState.isActive() or self.stuckCloseState.isActive() :                      #S6b
            return 1. # CONTINUE                                                                    #S6b
                                                                                                    #S6b
        if self.openState.isActive () and self.pi_tankLevel.dValue(0) > self.po_levelMax.dValue():  #S6b
            return -1. # STOP                                                                       #S6b
                                                                                                    #S6b
        if self.closeState.isActive() and self.pi_tankLevel.dValue(0) < self.po_levelMin.dValue():  #S6b
            return -1. # STOP                                                                       #S6b
                                                                                                    #S6b
        return 1. # CONTINUE                                                                        #S6b


########################################################################################
class Valve(HydroDevice):                                                                       #S2a ==> S2b
    def __init__(self, name)    :                                                               #S2a
        HydroDevice.__init__(self, name)                                                        #S2a ==> S2b


    def openCondition(self):                                                                    #S6a
        return self.pi_tankLevel.dValue(0) > self.po_levelMax.dValue()                          #S6a

    def closeCondition(self):                                                                   #S6a
            return (self.pi_tankLevel.dValue(0) < self.po_levelMin.dValue())                    #S6a


    def boundaryChecker(self):                                                                      #S6b
        if self.stuckOpenState.isActive() or self.stuckCloseState.isActive():                       #S6b
            return 1. # CONTINUE                                                                    #S6b
                                                                                                    #S6b
        if self.openState.isActive () and self.pi_tankLevel.dValue(0) < self.po_levelMin.dValue():  #S6b
            return -1. # STOP                                                                       #S6b
                                                                                                    #S6b
        if self.closeState.isActive() and self.pi_tankLevel.dValue(0) > self.po_levelMax.dValue():  #S6b
            return -1. # STOP                                                                       #S6b
                                                                                                    #S6b
        return 1. # CONTINUE                                                                        #S6b


########################################################################################
class Fuel(Pyc.CComponent):                                                                         #S7
    def __init__(self, name):                                                                       #S7
        Pyc.CComponent.__init__(self, name)                                                         #S7

        self.po_power = self.addVariable("power", Pyc.TVarType.t_double, 5.)                           #S7
        self.addMessageBox("Tank")                                                                  #S7
        self.addMessageBoxExport("Tank", self.po_power, "power")                                    #S7


class MySystem(Pyc.CSystem):                                                                    #S1
    def __init__(self, name):                                                                   #S1
        Pyc.CSystem.__init__(self, name)                                                        #S1

        self.P1   = Pump("P1")                                                                  #S1
        self.P2   = Pump("P2")                                                                  #S1
        self.V3   = Valve("V3")                                                                 #S2a
        self.tank = Tank("Tank")                                                                #S1
        self.fuel = Fuel("Fuel")                                                                #S7

        self.connect("P1"  , "Tank", "Tank", "Pump" )                                           #S1
        self.connect("P2"  , "Tank", "Tank", "Pump" )                                           #S1
        self.connect("V3"  , "Tank", "Tank", "Valve")                                           #S2a
        self.connect("Fuel", "Tank", "Tank", "Fuel" )                                           #S7




def mySequenceFilter(sequence):                                                                 #S8d
    ofElement = Pyc.CSystem.glSystem().getMonitoredElt("Tank.OverFlow"      , "ST")             #S8d
    doElement = Pyc.CSystem.glSystem().getMonitoredElt("Tank.DryOut"        , "ST")             #S8d
    htElement = Pyc.CSystem.glSystem().getMonitoredElt("Tank.HotTemperature", "ST")             #S8d

    return( sequence.value(ofElement, system.tMax()) == 1 or                                    #S8d
            sequence.value(doElement, system.tMax()) == 1 or                                    #S8d
            sequence.value(htElement, system.tMax()) == 1                                       #S8d
          )




if __name__ == '__main__':                                                                      #S1

    print("PyCATSHOO Version : ", Pyc.ILogManager.glLogManager().version())

    system = MySystem("HeatedTankSystem")                                                       #S1
    system.loadParameters("S8dHTBase.xml")                                                           #S1
    system.setRNGSeed(4536)                                                                     #S1

    system.setResultFileName("S8dHTBaseRes.xml", True)

    system.addTarget("HotTemperature", "Tank.HotTemperature", "ST")                             #S8c
    system.addTarget("OverFlow"      , "Tank.OverFlow"      , "ST")                             #S8c
    system.addTarget("DryOut"        , "Tank.DryOut"        , "ST")                             #S8c


    system.addInstants(0, system.tMax(), 100)                                                   #S8c


    ofIndicator = system.addIndicator("Overflow", "Tank.OverFlow", "ST")                        #S8c
    ofIndicator.setRestitutions(Pyc.TIndicatorType.mean_values)                                     #S8c

    doIndicator = system.addIndicator("DryOut", "Tank.DryOut", "ST")                          #S8c
    doIndicator.setRestitutions(Pyc.TIndicatorType.mean_values)                                 #S8c


    htIndicator = system.addIndicator("HotTemp", "Tank.HotTemperature", "ST")                   #S8c
    htIndicator.setRestitutions(Pyc.TIndicatorType.mean_values)                                 #S8c



    system.setSeqFilter(mySequenceFilter)                                                       #S8d
    system.setKeepFilteredSeqForInd(True)                                                       #S8d

    tb = time.time()                                                                            #S1
    system.simulate()                                                                           #S1

    if system.MPIRank() > 0 :
        exit(0)

    te=time .time()                                                                             #S1
    sTime = te-tb                                                                               #S1
    print ("Simulation time ", sTime )                                                          #S1

    meanOVF = ofIndicator.means()                                                               #S8c
    meanDO  = doIndicator.means()                                                               #S8c
    meanHT  = htIndicator.means()                                                               #S8c

    tps = system.instants()                                                                     #S8c


    plot(tps, True, True,                                                                      #S5 ==> S8a
            (                                                                                  #S1
                (("Tank"), "Time", "OverFlow", ((meanOVF, 'r-', 'Tank OverFlow'),)),           # S8a
                (("Tank"), "Time", "DryOut"  , ((meanDO , 'g-', 'Tank DryOut'  ),)),           # S8a
                (("Tank"), "Time", "Burnt"   , ((meanHT , 'b-', 'Tank Burnt'   ),)),           # S8a
            )                                                                                  #S1
         )                                                                                     #S1





    analyzer = Pyc.CAnalyser(system)                                                           #S8d
    if not analyzer.printFilteredSeq(10., "S8eHTBaseSeq.xml", "PySeq.xsl"):                    #S8d
        Pyc.ILogManager.glLogManager().printMessages()                                         #S8d


